---
title: Автоматическая очистка хоста
permalink: usage/cleanup/host_cleanup.html
---

Очистка хоста удаляет неактуальных данные и сокращает размер кеша **автоматически** в рамках вызова основных команд werf и сразу **для всех проектов**. При необходимости очистку можно выполнять в ручном режиме с помощью команды [**werf host cleanup**]({{ "reference/cli/werf_host_cleanup.html" | true_relative_url }}).

## Принцип работы

Алгоритм автоматически за пользователя решает какие данные будут удалены. Алгоритм очистки может быть представлен следующими шагами:

- Оценивается используемое место на том томе, где располагаются данные локального docker server.
- Если используемое место превышает выставленный порог (по умолчанию это 70% занятого места на томе — задаётся параметром `--allowed-docker-storage-volume`), то вычисляется количество места, которое требуется освободить, чтобы снизить используемое место на томе до выставленного уровня (по умолчанию это порог в 70% минус 5% запаса — задаётся параметром `--allowed-docker-storage-volume-usage`).
- Далее алгоритм начинает удалять наиболее давно используемые данные (по принципу LRU — least recently used) до тех пор пока не будет удалено достаточно данных, чтобы уровень занятого места на диске снизился до требуемого порога (по умолчанию это порог в 70% минус 5% запаса — задаются параметрами `--allowed-local-cache-volume-usage` и `--allowed-local-cache-volume-usage-margin` соответственно).

Какие данные могут быть удалены:
- Git-архивы из локального кеша werf: `~/.werf/local_cache/git_archives`.
- Git-патчи из локального кеша werf: `~/.werf/local_cache/git_patches`.
- Git-репозитории из локального кеша werf: `~/.werf/local_cache/git_repos`.
- Git-worktree из локального кеша werf: `~/.werf/local_cache/git_worktrees`.
- Все docker образы, собранные версией v1.2, оставшиеся в локальном docker server.

Алгоритм применяется отдельно для тома где хранится локальный кеш werf `~/.werf/local_cache` и для тома, где хранятся данные локального docker server (обычно это `/var/lib/docker`). В том случае, если werf не может самостоятельно определить том, где хранятся реальные данные локального docker server, имеется возможность явно указать директорию данных локального docker server через параметр `--docker-server-storage-path=/var/lib/docker` (либо через переменную окружения `WERF_DOCKER_SERVER_STORAGE_PATH`).

## Выключение автоматической очистки

Пользователь может выключить автоматическую очистку неактуальных данных хоста с помощью параметра `--disable-auto-host-cleanup` (или переменной окружения `WERF_DISABLE_AUTO_HOST_CLEANUP`). В этом случае рекомендуется добавить команду `werf host cleanup` в cron, например следующим образом:

```shell
# /etc/cron.d/werf-host-cleanup
SHELL=/bin/bash
*/30 * * * * gitlab-runner source ~/.profile ; source $(trdl use werf 1.2 stable) ; werf host cleanup
```
